import 'dart:async';
import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:flutter_socket_io/iflutter_socket_io.dart';

class SocketIO implements ISocketIO {
  static const MethodChannel _channel =
      const MethodChannel('flutter_socket_io');

  String _domain;
  String _namespace;
  String _statusCallback;

  SocketIO(String domain, String namespace, String statusCallback,
      Future<dynamic> handler(MethodCall call)) {
    _domain = domain;
    _namespace = namespace;
    _statusCallback = statusCallback;
    _channel.setMethodCallHandler(handler);
  }

  @override
  void connect() async {
    await _channel.invokeMethod(MethodCallName.SOCKET_CONNECT, {
      MethodCallArgumentsName.SOCKET_DOMAIN: _domain,
      MethodCallArgumentsName.SOCKET_NAME_SPACE: _namespace,
      MethodCallArgumentsName.SOCKET_CALLBACK: _statusCallback
    });
  }

  /// Map subscribes (Map<String, String>): key is event name, value [optional] is callback event function
  ///
  @override
  void subscribes(Map<String, String> subscribes) async {
    await _channel.invokeMethod(MethodCallName.SOCKET_SUBSCRIBES, {
      MethodCallArgumentsName.SOCKET_DOMAIN: _domain,
      MethodCallArgumentsName.SOCKET_NAME_SPACE: _namespace,
      MethodCallArgumentsName.SOCKET_DATA: json.encode(subscribes),
    });
  }

  @override
  void sendMessage(String event, dynamic message, [String callback]) async {
    print(message);
    await _channel.invokeMethod(MethodCallName.SOCKET_SEND_MESSAGE, {
      MethodCallArgumentsName.SOCKET_DOMAIN: _domain,
      MethodCallArgumentsName.SOCKET_NAME_SPACE: _namespace,
      MethodCallArgumentsName.SOCKET_EVENT: event,
      MethodCallArgumentsName.SOCKET_MESSAGE: message,
      MethodCallArgumentsName.SOCKET_CALLBACK: callback
    });
  }

  /// Map unsubscribes: key is event name, value [optional] is callback event function
  ///
  @override
  void unSubscribes(Map<String, String> unsubscribes) async {
    await _channel.invokeMethod(MethodCallName.SOCKET_UNSUBSCRIBES, {
      MethodCallArgumentsName.SOCKET_DOMAIN: _domain,
      MethodCallArgumentsName.SOCKET_NAME_SPACE: _namespace,
      MethodCallArgumentsName.SOCKET_DATA: json.encode(unsubscribes),
    });
  }

  @override
  void disconnect() async {
    await _channel.invokeMethod(MethodCallName.SOCKET_DISCONNECT, {
      MethodCallArgumentsName.SOCKET_DOMAIN: _domain,
      MethodCallArgumentsName.SOCKET_NAME_SPACE: _namespace,
    });
  }

  @override
  String getId() {
    if (_domain != null) {
      return _domain + (_namespace != null ? _namespace : "");
    }
    return null;
  }

  @override
  void destroy() async {
    await _channel.invokeMethod(MethodCallName.SOCKET_DESTROY, {
      MethodCallArgumentsName.SOCKET_DOMAIN: _domain,
      MethodCallArgumentsName.SOCKET_NAME_SPACE: _namespace
    });
  }

  @override
  void unSubscribesAll() async {
    await _channel.invokeMethod(MethodCallName.SOCKET_UNSUBSCRIBES_ALL, {
      MethodCallArgumentsName.SOCKET_DOMAIN: _domain,
      MethodCallArgumentsName.SOCKET_NAME_SPACE: _namespace
    });
  }
}

class MethodCallArgumentsName {
  static final String SOCKET_DOMAIN = "socketDomain";
  static final String SOCKET_NAME_SPACE = "socketNameSpace";
  static final String SOCKET_CALLBACK = "socketCallback";
  static final String SOCKET_EVENT = "socketEvent";
  static final String SOCKET_MESSAGE = "socketMessage";
  static final String SOCKET_DATA = "socketData";
}

class MethodCallName {
  static final String SOCKET_CONNECT = "socketConnect";
  static final String SOCKET_DISCONNECT = "socketDisconnect";
  static final String SOCKET_SUBSCRIBES = "socketSubcribes";
  static final String SOCKET_UNSUBSCRIBES = "socketUnsubcribes";
  static final String SOCKET_UNSUBSCRIBES_ALL = "socketUnsubcribesAll";
  static final String SOCKET_SEND_MESSAGE = "socketSendMessage";
  static final String SOCKET_DESTROY = "socketDestroy";
  static final String SOCKET_DESTROY_ALL = "socketDestroyAll";
}
